attribute vec4 a_position;
attribute vec2 a_uv;

uniform vec2 u_noiseShift;
uniform float u_noiseAmount;
uniform sampler2D u_texture;
uniform mat4 u_projectionmodel;
uniform vec2 u_gridShift;
uniform vec2 u_forward1;
uniform vec2 u_forward2;
uniform float u_time;

varying float v_roadness;
varying float v_glimmer;
varying vec2 v_uv;

const vec2 noiseCenter = vec2(7.0/8.0, 7.0/8.0);
const vec2 gridStep = vec2(10.0/80.0);
const float highfreqRatio = 1.0/8.0;
const float lowfreqRatio = 1.0/12.0;

void main(void)
{
    v_uv = a_uv;
    
    vec4 position = a_position;
    vec2 gridShift = mod(u_gridShift, gridStep);
    position.xz += gridShift.yx;
    
    vec2 reverseShift = -u_gridShift.yx;
    reverseShift /= 10.0;
    
    vec2 glimmerPos = position.xz/10.0;
    glimmerPos += u_time;
    glimmerPos = mod(glimmerPos + 1.0, vec2(2.0)) - 1.0;
    glimmerPos *= highfreqRatio;
    v_glimmer = 1.0 - texture2D(u_texture, noiseCenter + glimmerPos).r;
    
    vec2 noisePos = position.xz/10.0;
    noisePos += u_noiseShift;
    noisePos += reverseShift;
    noisePos = abs(mod(noisePos + 1.0, vec2(2.0)) - 1.0) * 2.0 - 1.0;
    noisePos *= highfreqRatio;
    float noise = texture2D(u_texture, noiseCenter + noisePos).r;
    
    noisePos *= lowfreqRatio;
    noise += 2.0 * texture2D(u_texture, noiseCenter + noisePos).r;
    noise -= 1.4;
    
    float ditchingFactor = abs(dot(position.xz, mix(u_forward1, u_forward2, length(position.xz)) ));
    ditchingFactor = clamp(ditchingFactor * 2.0 - 0.6, 0.07, 1.0);
    noise *= ditchingFactor;
    
    position.y += noise * u_noiseAmount;
    v_roadness = 1.0 - ditchingFactor;
    gl_Position = u_projectionmodel * position;
}